import os
import json
import subprocess
import shutil
from datetime import datetime
from argparse import ArgumentParser
from ftplib import FTP, error_perm
from webdav3.client import Client as WebDavClient

def load_config():
    with open('config.json', 'r') as file:
        return json.load(file)

def load_stations():
    with open('stations.json', 'r') as file:
        return json.load(file)

def manage_ftp_space(ftp, folder_path, config):
    ftp_max_files = config['ftp_config']['ftp_max_files']

    # Jeśli max_files jest równy 0, nie usuwaj żadnych plików
    if ftp_max_files == 0:
        return

    ftp.cwd(folder_path)
    files = ftp.nlst()
    audio_files = [file for file in files if file.endswith('.mp3') or file.endswith('.opus')]

    while len(audio_files) > ftp_max_files:
        oldest_file = min(audio_files, key=lambda x: ftp.sendcmd('MDTM ' + x))
        ftp.delete(oldest_file)
        print(f"Usunięto najstarszy plik: {oldest_file}")
        audio_files.remove(oldest_file)

def manage_nextcloud_space(client, folder_path, config, station, show_name):
    nextcloud_max_files = config['nextcloud_config']['nextcloud_max_files']
    if nextcloud_max_files == 0:
        return

    # Użyj argumentów station i show_name zamiast tworzyć ścieżkę wewnątrz funkcji
    full_folder_path = folder_path  # Ścieżka jest już przekazywana jako argument

    # Pobranie listy plików z folderu
    files = client.list(full_folder_path)

    # Filtrowanie tylko plików .mp3 lub .opus
    audio_files = [file for file in files if file.endswith('.mp3') or file.endswith('.opus')]

    # Budowanie pełnych ścieżek dla filtrowanych plików
    full_files_paths = [os.path.join(full_folder_path, file) for file in audio_files]

    # Pobieranie informacji o plikach (może wymagać dostosowania w zależności od API klienta WebDAV)
    files_info = [(file, client.info(file)) for file in full_files_paths]

    # Sortowanie plików po dacie modyfikacji i usuwanie najstarszych, jeśli przekraczają limit
    if len(files_info) > nextcloud_max_files:
        files_sorted = sorted(files_info, key=lambda x: x[1].get('modified'))
        for file_path, _ in files_sorted[:-nextcloud_max_files]:
            # Użycie metody clean do usunięcia pliku
            client.clean(file_path)
            print(f"Usunięto najstarszy plik w Nextcloud: {file_path}")


def upload_ftp(filename, config, station, show_name):
    with FTP(config['ftp_config']['ftp_hostname']) as ftp:
        ftp.login(config['ftp_config']['ftp_username'], config['ftp_config']['ftp_password'])
        remote_dir = f"{config['ftp_config']['ftp_base_path']}/{station}/{show_name}"
        try:
            ftp.cwd(remote_dir)
        except error_perm:
            ftp.mkd(remote_dir)
            ftp.cwd(remote_dir)
        with open(filename, 'rb') as file:
            ftp.storbinary('STOR ' + os.path.basename(filename), file)
        manage_ftp_space(ftp, remote_dir, config)
        print(f"File {filename} successfully uploaded to FTP.")

def ensure_remote_path(client, remote_path):
    parts = remote_path.strip('/').split('/')
    path = ''
    for part in parts[:-1]:
        path += f"/{part}"
        if not client.check(path):
            client.mkdir(path)

def upload_nextcloud(filename, config, station, show_name):
    options = {
        'webdav_hostname': config['nextcloud_config']['nextcloud_hostname'],  # Poprawiona ścieżka
        'webdav_login': config['nextcloud_config']['nextcloud_login'],  # Poprawiona ścieżka
        'webdav_password': config['nextcloud_config']['nextcloud_password']  # Poprawiona ścieżka
    }
    client = WebDavClient(options)
    remote_dir = f"/public/{station}/{show_name}/"
    remote_path = remote_dir + os.path.basename(filename)

    # Upewnij się, że katalog nadrzędny istnieje
    ensure_remote_path(client, remote_path)

    client.upload_sync(remote_path=remote_path, local_path=filename)
    manage_nextcloud_space(client, remote_dir, config, station, show_name)
    print(f"File {filename} successfully uploaded to NextCloud.")


def record_show(duration, station_url, station, show_name, config):
    local_audio_config = config['local_audio_config']
    date_str = datetime.now().strftime("%Y_%m_%d_%H_%M")
    output_dir = f"./recordings/{station}/{show_name}/"
    os.makedirs(output_dir, exist_ok=True)
    output_filename = f"{output_dir}{station}_{show_name}_{date_str}.{local_audio_config['local_audio_format']}"
    ffmpeg_command = [
        "ffmpeg",
        "-t", str(duration),
        "-i", station_url,
        "-acodec", "libopus" if local_audio_config['local_audio_format'] == 'opus' else "libmp3lame",
        "-b:a", f"{local_audio_config['local_audio_bitrate']}k",
        "-ac", str(local_audio_config['local_audio_channels']),
        "-y", output_filename
    ]
    subprocess.run(ffmpeg_command)
    return output_filename

def convert_audio(source_file, config, destination):
    # Pobieranie konfiguracji w zależności od docelowego miejsca przesyłania
    target_config = config['ftp_config'] if destination == 'ftp' else config['nextcloud_config']

    # Pobieranie parametrów docelowych
    target_format = target_config['ftp_audio_format'] if destination == 'ftp' else target_config['nextcloud_audio_format']
    target_bitrate = target_config['ftp_audio_bitrate'] if destination == 'ftp' else target_config['nextcloud_audio_bitrate']
    channels = target_config['ftp_audio_channels'] if destination == 'ftp' else target_config['nextcloud_audio_channels']

    # Określenie nazwy pliku docelowego
    target_filename = os.path.splitext(source_file)[0] + f'_{destination}.{target_format}'

    # Sprawdzanie, czy konwersja jest potrzebna
    if not (config['local_audio_config']['local_audio_format'] == target_format and
            config['local_audio_config']['local_audio_bitrate'] == target_bitrate and
            config['local_audio_config']['local_audio_channels'] == channels):
        print(f"Konwertowanie pliku do formatu {target_format} z bitrate: {target_bitrate}k i {channels} kanałów.")
        codec = "libopus" if target_format == 'opus' else "libmp3lame"
        ffmpeg_command = [
            "ffmpeg",
            "-i", source_file,
            "-acodec", codec,
            "-b:a", f"{target_bitrate}k",
            "-ac", str(channels),
            "-f", target_format,
            "-y", target_filename
        ]
        subprocess.run(ffmpeg_command, check=True)
    else:
        print("Plik wejściowy jest już w docelowym formacie i jakości. Tworzenie kopii.")
        shutil.copy(source_file, target_filename)

    return target_filename

def main():
    parser = ArgumentParser()
    parser.add_argument("duration", type=int, help="Duration of the recording in minutes")
    parser.add_argument("station", type=str, help="Station name")
    parser.add_argument("show_name", type=str, help="Name of the show")
    parser.add_argument("--upload", nargs='+', choices=['ftp', 'nextcloud'], help="Upload destinations", default=[])
    parser.add_argument("--keep_local", type=str, choices=['yes', 'no'], default='yes', help="Whether to keep the file locally")
    args = parser.parse_args()

    config = load_config()
    stations = load_stations()
    station_url = stations[args.station]

    original_file = record_show(args.duration, station_url, args.station, args.show_name, config)

    if 'ftp' in args.upload:
        ftp_file = convert_audio(original_file, config, 'ftp')
        upload_ftp(ftp_file, config, args.station, args.show_name)
        # Zawsze usuwaj plik po wysłaniu na FTP
        os.remove(ftp_file)

    if 'nextcloud' in args.upload:
        nextcloud_file = convert_audio(original_file, config, 'nextcloud')
        upload_nextcloud(nextcloud_file, config, args.station, args.show_name)
        if nextcloud_file != original_file:
            os.remove(nextcloud_file)

    # Usuwanie oryginalnego pliku, jeśli --keep_local=no, niezależnie od tego, czy wysyłamy na FTP/NextCloud
    if args.keep_local == 'no':
        os.remove(original_file)

if __name__ == "__main__":
    main()
